==> flirimageextractor-1.2.6-py2-none-any.whl: flirimageextractor/__init__.py <==
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# version 1.2.2

from __future__ import print_function

import argparse
import io
import json
import os
import os.path
import re
import csv
import subprocess
from PIL import Image, ImageEnhance
from math import sqrt, exp, log
from matplotlib import cm
from matplotlib import pyplot as plt
from io import StringIO
from io import BytesIO
import numpy as np


class FlirImageExtractor:

    def __init__(self, exiftool_path="exiftool", is_debug=False, palettes=[cm.bwr, cm.gnuplot2, cm.gist_ncar]):
        self.exiftool_path = exiftool_path
        self.is_debug = is_debug
        self.flir_img_filename = None
        self.flir_img_bytes = None
        self.image_suffix = "_rgb_image.jpg"
        self.thumbnail_suffix = "_rgb_thumb.jpg"
        self.thermal_suffix = "_thermal.jpg"
        self.default_distance = 1.0

        # valid for PNG thermal images
        self.fix_endian = True
        self.use_thumbnail = False

        self.rgb_image_np = None
        self.thermal_image_np = None

        self.palettes = palettes

    pass

    # def get_metadata(self, flir_img_file, bytesIO=False):
    #     """
    #     Given a valid image path, get relevant metadata out of the image using exiftool
    #     :param flir_img_file: Input path for the flir image
    #     :return: dictionary of metadata
    #     """
    #     if bytesIO:
    #         self.flir_img_bytes = flir_img_file
    #     else:
    #         if self.is_debug:
    #             print("INFO Flir image filepath:{}".format(flir_img_file))
    #
    #         if not os.path.isfile(flir_img_file):
    #             raise ValueError("Input file does not exist or this user don't have permission on this file")
    #
    #         self.flir_img_filename = flir_img_file
    #
    #     if self.flir_img_filename:
    #         meta_json = subprocess.check_output([self.exiftool_path, self.flir_img_filename, '-j'])
    #     else:
    #         args = ['exiftool', '-j', '-']
    #         p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    #         meta_json, err = p.communicate(input=self.flir_img_bytes.read())
    #
    #     meta = json.loads(meta_json.decode())[0]
    #
    #     return meta
    #
    # def check_for_thermal_image(self, flir_img_filename):
    #     """
    #     Given a valid image path, return a boolean of whether the image contains thermal data
    #     :param flir_img_filename: Input path for the flir image
    #     :return: Bool
    #     """
    #     metadata = self.get_metadata(flir_img_filename)
    #     return 'RawThermalImageType' in metadata

    def process_image(self, flir_img_file, RGB=False, bytesIO=False):
        """
        Given a valid image path, process the file: extract real thermal values
        and a thumbnail for comparison (generally thumbnail is on the visible spectre)
        :param flir_img_file: Input path for the flir image
        :param RGB: Boolean for whether to extract the embedded RGB image
        :return:
        """
        if bytesIO:
            self.flir_img_bytes = flir_img_file
            # print(self.flir_img_bytes.read())
        else:
            if self.is_debug:
                print("INFO Flir image filepath:{}".format(flir_img_file))

            if not os.path.isfile(flir_img_file):
                raise ValueError("Input file does not exist or this user don't have permission on this file")

            self.flir_img_filename = flir_img_file


        if self.get_image_type().upper().strip() == "TIFF":
            # valid for tiff images from Zenmuse XTR
            self.use_thumbnail = True
            self.fix_endian = False

        self.thermal_image_np = self.extract_thermal_image()

        if RGB:
            self.rgb_image_np = self.extract_embedded_image()

    def get_image_type(self):
        """
        Get the embedded thermal image type, generally can be TIFF or PNG
        :return:
        """
        if self.flir_img_filename:
            meta_json = subprocess.check_output(
                [self.exiftool_path, '-RawThermalImageType', '-j', self.flir_img_filename])
        else:
            self.flir_img_bytes.seek(0)
            args = ['exiftool', '-RawThermalImageType', '-j', '-']
            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
            meta_json, err = p.communicate(input=self.flir_img_bytes.read())

        meta = json.loads(meta_json.decode())[0]

        return meta['RawThermalImageType']

    def get_rgb_np(self):
        """
        Return the last extracted rgb image
        :return:
        """
        return self.rgb_image_np

    def get_thermal_np(self):
        """
        Return the last extracted thermal image
        :return:
        """
        return self.thermal_image_np

    def extract_embedded_image(self):
        """
        extracts the visual image as 2D numpy array of RGB values
        """
        image_tag = "-EmbeddedImage"

        if self.flir_img_filename:
            visual_img_bytes = subprocess.check_output([self.exiftool_path, image_tag, "-b", self.flir_img_filename])
        else:
            self.flir_img_bytes.seek(0)
            args = ['exiftool', image_tag, "-b", '-']
            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
            visual_img_bytes, err = p.communicate(input=self.flir_img_bytes.read())

        visual_img_stream = io.BytesIO(visual_img_bytes)
        visual_img_stream.seek(0)

        visual_img = Image.open(visual_img_stream)
        visual_np = np.array(visual_img)

        return visual_np

    def extract_thermal_image(self):
        """
        extracts the thermal image as 2D numpy array with temperatures in oC
        """

        # read image metadata needed for conversion of the raw sensor values
        # E=1,SD=1,RTemp=20,ATemp=RTemp,IRWTemp=RTemp,IRT=1,RH=50,PR1=21106.77,PB=1501,PF=1,PO=-7340,PR2=0.012545258

        if self.flir_img_filename:
            meta_json = subprocess.check_output(
                [self.exiftool_path, self.flir_img_filename, '-Emissivity', '-SubjectDistance',
                 '-AtmosphericTemperature',
                 '-ReflectedApparentTemperature', '-IRWindowTemperature', '-IRWindowTransmission', '-RelativeHumidity',
                 '-PlanckR1', '-PlanckB', '-PlanckF', '-PlanckO', '-PlanckR2', '-j'])
        else:
            self.flir_img_bytes.seek(0)
            args = ['exiftool', '-Emissivity', '-SubjectDistance',
                 '-AtmosphericTemperature',
                 '-ReflectedApparentTemperature', '-IRWindowTemperature', '-IRWindowTransmission', '-RelativeHumidity',
                 '-PlanckR1', '-PlanckB', '-PlanckF', '-PlanckO', '-PlanckR2', '-j', '-']
            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
            meta_json, err = p.communicate(input=self.flir_img_bytes.read())

        meta = json.loads(meta_json.decode())[0]

        # exifread can't extract the embedded thermal image, use exiftool instead
        if self.flir_img_filename:
            thermal_img_bytes = subprocess.check_output(
                [self.exiftool_path, "-RawThermalImage", "-b", self.flir_img_filename])
        else:
            self.flir_img_bytes.seek(0)
            args = ['exiftool', "-RawThermalImage", "-b", '-']
            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
            thermal_img_bytes, err = p.communicate(input=self.flir_img_bytes.read())

        thermal_img_stream = io.BytesIO(thermal_img_bytes)
        thermal_img_stream.seek(0)

        thermal_img = Image.open(thermal_img_stream)
        thermal_np = np.array(thermal_img)

        # raw values -> temperature
        subject_distance = self.default_distance
        if 'SubjectDistance' in meta:
            subject_distance = FlirImageExtractor.extract_float(meta['SubjectDistance'])

        if self.fix_endian:
            # fix endianness, the bytes in the embedded png are in the wrong order
            thermal_np = np.vectorize(lambda x: (x >> 8) + ((x & 0x00ff) << 8))(thermal_np)

        raw2tempfunc = np.vectorize(lambda x: FlirImageExtractor.raw2temp(x, E=meta['Emissivity'], OD=subject_distance,
                                                                          RTemp=FlirImageExtractor.extract_float(
                                                                              meta['ReflectedApparentTemperature']),
                                                                          ATemp=FlirImageExtractor.extract_float(
                                                                              meta['AtmosphericTemperature']),
                                                                          IRWTemp=FlirImageExtractor.extract_float(
                                                                              meta['IRWindowTemperature']),
                                                                          IRT=meta['IRWindowTransmission'],
                                                                          RH=FlirImageExtractor.extract_float(
                                                                              meta['RelativeHumidity']),
                                                                          PR1=meta['PlanckR1'], PB=meta['PlanckB'],
                                                                          PF=meta['PlanckF'],
                                                                          PO=meta['PlanckO'], PR2=meta['PlanckR2']))
        thermal_np = raw2tempfunc(thermal_np)
        return thermal_np

    @staticmethod
    def raw2temp(raw, E=1, OD=1, RTemp=20, ATemp=20, IRWTemp=20, IRT=1, RH=50, PR1=21106.77, PB=1501, PF=1, PO=-7340,
                 PR2=0.012545258):
        """
        convert raw values from the flir sensor to temperatures in C
        # this calculation has been ported to python from
        # https://github.com/gtatters/Thermimage/blob/master/R/raw2temp.R
        # a detailed explanation of what is going on here can be found there
        """

        # constants
        ATA1 = 0.006569
        ATA2 = 0.01262
        ATB1 = -0.002276
        ATB2 = -0.00667
        ATX = 1.9

        # transmission through window (calibrated)
        emiss_wind = 1 - IRT
        refl_wind = 0

        # transmission through the air
        h2o = (RH / 100) * exp(1.5587 + 0.06939 * (ATemp) - 0.00027816 * (ATemp) ** 2 + 0.00000068455 * (ATemp) ** 3)
        tau1 = ATX * exp(-sqrt(OD / 2) * (ATA1 + ATB1 * sqrt(h2o))) + (1 - ATX) * exp(
            -sqrt(OD / 2) * (ATA2 + ATB2 * sqrt(h2o)))
        tau2 = ATX * exp(-sqrt(OD / 2) * (ATA1 + ATB1 * sqrt(h2o))) + (1 - ATX) * exp(
            -sqrt(OD / 2) * (ATA2 + ATB2 * sqrt(h2o)))

        # radiance from the environment
        raw_refl1 = PR1 / (PR2 * (exp(PB / (RTemp + 273.15)) - PF)) - PO
        raw_refl1_attn = (1 - E) / E * raw_refl1
        raw_atm1 = PR1 / (PR2 * (exp(PB / (ATemp + 273.15)) - PF)) - PO
        raw_atm1_attn = (1 - tau1) / E / tau1 * raw_atm1
        raw_wind = PR1 / (PR2 * (exp(PB / (IRWTemp + 273.15)) - PF)) - PO
        raw_wind_attn = emiss_wind / E / tau1 / IRT * raw_wind
        raw_refl2 = PR1 / (PR2 * (exp(PB / (RTemp + 273.15)) - PF)) - PO
        raw_refl2_attn = refl_wind / E / tau1 / IRT * raw_refl2
        raw_atm2 = PR1 / (PR2 * (exp(PB / (ATemp + 273.15)) - PF)) - PO
        raw_atm2_attn = (1 - tau2) / E / tau1 / IRT / tau2 * raw_atm2
        raw_obj = (raw / E / tau1 / IRT / tau2 - raw_atm1_attn -
                   raw_atm2_attn - raw_wind_attn - raw_refl1_attn - raw_refl2_attn)

        # temperature from radiance
        temp_celcius = PB / log(PR1 / (PR2 * (raw_obj + PO)) + PF) - 273.15
        return temp_celcius

    @staticmethod
    def extract_float(dirtystr):
        """
        Extract the float value of a string, helpful for parsing the exiftool data
        :return:
        """
        digits = re.findall(r"[-+]?\d*\.\d+|\d+", dirtystr)
        return float(digits[0])

    def plot(self):
        """
        Plot the rgb + thermal image (easy to see the pixel values)
        :return:
        """
        rgb_np = self.get_rgb_np()
        thermal_np = self.get_thermal_np()

        plt.subplot(1, 2, 1)
        plt.imshow(thermal_np, cmap='hot')
        plt.subplot(1, 2, 2)
        plt.imshow(rgb_np)
        plt.show()

    def save_images(self, min=None, max=None, bytesIO=False):
        """
        Save the extracted images
        :return:
        """
        if (min is not None and max is None) or (max is not None and min is None):
            raise Exception('Specify both a maximum and minimum temperature value, or use the default by specifying neither')
        if max is not None and min is not None and max <= min:
            raise Exception("The max value must be greater than min")
        thermal_np = self.extract_thermal_image()

        if min is not None and max is not None:
            thermal_normalized = ((thermal_np - min) / (max - min))
        else:
            thermal_normalized = ((thermal_np - np.amin(thermal_np)) / (np.amax(thermal_np) - np.amin(thermal_np)))

        if not bytesIO:
            thermal_output_filename_array = self.flir_img_filename.split(".")
            thermal_output_filename = thermal_output_filename_array[0] + "_thermal." + thermal_output_filename_array[1]

        return_array = []
        for palette in self.palettes:
            img_thermal = Image.fromarray(palette(thermal_normalized, bytes=True))
            # convert to jpeg and enhance
            img_thermal = img_thermal.convert("RGB")
            enhancer = ImageEnhance.Sharpness(img_thermal)
            img_thermal = enhancer.enhance(3)

            if bytesIO:
                bytes = io.BytesIO()
                img_thermal.save(bytes, "jpeg", quality=100)
                return_array.append(bytes)
            else:
                filename_array = thermal_output_filename.split(".")
                filename = filename_array[0] + "_" + str(palette.name) + "." + filename_array[1]
                if self.is_debug:
                    print("DEBUG Saving Thermal image to:{}".format(filename))

                img_thermal.save(filename, "jpeg", quality=100)
                return_array.append(filename)

            return return_array

    def export_thermal_to_csv(self, csv_filename):
        """
        Convert thermal data in numpy to csv
        :return:
        """
        with open(csv_filename, 'w') as fh:
            writer = csv.writer(fh, delimiter=',')

            pixel_values = [[]]
            current_x = 0
            for e in np.ndenumerate(self.thermal_image_np):
                x, y = e[0]

                if x > current_x:
                    pixel_values.append([])
                    current_x = x

                c = e[1].round(2)
                pixel_values[x].append(c)

            writer.writerows(pixel_values)

    @staticmethod
    def reject_outliers(data, m=2):
        return data[abs(data - np.mean(data)) < m * np.std(data)]

==> flirimageextractor-1.2.7-py2-none-any.whl: flirimageextractor/__init__.py <==
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# version 1.2.2

from __future__ import print_function

import argparse
import io
import json
import os
import os.path
import re
import csv
import subprocess
from PIL import Image, ImageEnhance
from math import sqrt, exp, log
from matplotlib import cm
from matplotlib import pyplot as plt
from io import StringIO
from io import BytesIO
import numpy as np


class FlirImageExtractor:

    def __init__(self, exiftool_path="exiftool", is_debug=False, palettes=[cm.bwr, cm.gnuplot2, cm.gist_ncar]):
        self.exiftool_path = exiftool_path
        self.is_debug = is_debug
        self.flir_img_filename = None
        self.flir_img_bytes = None
        self.image_suffix = "_rgb_image.jpg"
        self.thumbnail_suffix = "_rgb_thumb.jpg"
        self.thermal_suffix = "_thermal.jpg"
        self.default_distance = 1.0

        # valid for PNG thermal images
        self.fix_endian = True
        self.use_thumbnail = False

        self.rgb_image_np = None
        self.thermal_image_np = None

        self.palettes = palettes

    pass

    # def get_metadata(self, flir_img_file, bytesIO=False):
    #     """
    #     Given a valid image path, get relevant metadata out of the image using exiftool
    #     :param flir_img_file: Input path for the flir image
    #     :return: dictionary of metadata
    #     """
    #     if bytesIO:
    #         self.flir_img_bytes = flir_img_file
    #     else:
    #         if self.is_debug:
    #             print("INFO Flir image filepath:{}".format(flir_img_file))
    #
    #         if not os.path.isfile(flir_img_file):
    #             raise ValueError("Input file does not exist or this user don't have permission on this file")
    #
    #         self.flir_img_filename = flir_img_file
    #
    #     if self.flir_img_filename:
    #         meta_json = subprocess.check_output([self.exiftool_path, self.flir_img_filename, '-j'])
    #     else:
    #         args = ['exiftool', '-j', '-']
    #         p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    #         meta_json, err = p.communicate(input=self.flir_img_bytes.read())
    #
    #     meta = json.loads(meta_json.decode())[0]
    #
    #     return meta
    #
    # def check_for_thermal_image(self, flir_img_filename):
    #     """
    #     Given a valid image path, return a boolean of whether the image contains thermal data
    #     :param flir_img_filename: Input path for the flir image
    #     :return: Bool
    #     """
    #     metadata = self.get_metadata(flir_img_filename)
    #     return 'RawThermalImageType' in metadata

    def process_image(self, flir_img_file, RGB=False, bytesIO=False):
        """
        Given a valid image path, process the file: extract real thermal values
        and a thumbnail for comparison (generally thumbnail is on the visible spectre)
        :param flir_img_file: Input path for the flir image
        :param RGB: Boolean for whether to extract the embedded RGB image
        :return:
        """
        if bytesIO:
            self.flir_img_bytes = flir_img_file
            # print(self.flir_img_bytes.read())
        else:
            if self.is_debug:
                print("INFO Flir image filepath:{}".format(flir_img_file))

            if not os.path.isfile(flir_img_file):
                raise ValueError("Input file does not exist or this user don't have permission on this file")

            self.flir_img_filename = flir_img_file

        if self.get_image_type().upper().strip() == "TIFF":
            # valid for tiff images from Zenmuse XTR
            self.use_thumbnail = True
            self.fix_endian = False

        self.thermal_image_np = self.extract_thermal_image()

        if RGB:
            self.rgb_image_np = self.extract_embedded_image()

    def get_image_type(self):
        """
        Get the embedded thermal image type, generally can be TIFF or PNG
        :return:
        """
        if self.flir_img_filename:
            meta_json = subprocess.check_output(
                [self.exiftool_path, '-RawThermalImageType', '-j', self.flir_img_filename])
        else:
            self.flir_img_bytes.seek(0)
            args = ['exiftool', '-RawThermalImageType', '-j', '-']
            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
            meta_json, err = p.communicate(input=self.flir_img_bytes.read())

        meta = json.loads(meta_json.decode())[0]

        return meta['RawThermalImageType']

    def get_rgb_np(self):
        """
        Return the last extracted rgb image
        :return:
        """
        return self.rgb_image_np

    def get_thermal_np(self):
        """
        Return the last extracted thermal image
        :return:
        """
        return self.thermal_image_np

    def extract_embedded_image(self):
        """
        extracts the visual image as 2D numpy array of RGB values
        """
        image_tag = "-EmbeddedImage"

        if self.flir_img_filename:
            visual_img_bytes = subprocess.check_output([self.exiftool_path, image_tag, "-b", self.flir_img_filename])
        else:
            self.flir_img_bytes.seek(0)
            args = ['exiftool', image_tag, "-b", '-']
            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
            visual_img_bytes, err = p.communicate(input=self.flir_img_bytes.read())

        visual_img_stream = io.BytesIO(visual_img_bytes)
        visual_img_stream.seek(0)

        visual_img = Image.open(visual_img_stream)
        visual_np = np.array(visual_img)

        return visual_np

    def extract_thermal_image(self):
        """
        extracts the thermal image as 2D numpy array with temperatures in oC
        """

        # read image metadata needed for conversion of the raw sensor values
        # E=1,SD=1,RTemp=20,ATemp=RTemp,IRWTemp=RTemp,IRT=1,RH=50,PR1=21106.77,PB=1501,PF=1,PO=-7340,PR2=0.012545258
        if self.flir_img_filename:
            meta_json = subprocess.check_output(
                [self.exiftool_path, self.flir_img_filename, '-Emissivity', '-SubjectDistance',
                 '-AtmosphericTemperature',
                 '-ReflectedApparentTemperature', '-IRWindowTemperature', '-IRWindowTransmission', '-RelativeHumidity',
                 '-PlanckR1', '-PlanckB', '-PlanckF', '-PlanckO', '-PlanckR2', '-j'])
        else:
            self.flir_img_bytes.seek(0)
            args = ['exiftool', '-Emissivity', '-SubjectDistance',
                 '-AtmosphericTemperature',
                 '-ReflectedApparentTemperature', '-IRWindowTemperature', '-IRWindowTransmission', '-RelativeHumidity',
                 '-PlanckR1', '-PlanckB', '-PlanckF', '-PlanckO', '-PlanckR2', '-j', '-']
            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
            meta_json, err = p.communicate(input=self.flir_img_bytes.read())

        meta = json.loads(meta_json.decode())[0]

        # exifread can't extract the embedded thermal image, use exiftool instead
        if self.flir_img_filename:
            thermal_img_bytes = subprocess.check_output(
                [self.exiftool_path, "-RawThermalImage", "-b", self.flir_img_filename])
        else:
            self.flir_img_bytes.seek(0)
            args = ['exiftool', "-RawThermalImage", "-b", '-']
            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
            thermal_img_bytes, err = p.communicate(input=self.flir_img_bytes.read())

        thermal_img_stream = io.BytesIO(thermal_img_bytes)
        thermal_img_stream.seek(0)

        thermal_img = Image.open(thermal_img_stream)
        thermal_np = np.array(thermal_img)

        # raw values -> temperature
        subject_distance = self.default_distance
        if 'SubjectDistance' in meta:
            subject_distance = FlirImageExtractor.extract_float(meta['SubjectDistance'])

        if self.fix_endian:
            # fix endianness, the bytes in the embedded png are in the wrong order
            thermal_np = np.vectorize(lambda x: (x >> 8) + ((x & 0x00ff) << 8))(thermal_np)

        thermal_np = FlirImageExtractor.raw2temp(self, thermal_np, E=meta['Emissivity'], OD=subject_distance,
                                                                          RTemp=FlirImageExtractor.extract_float(
                                                                              meta['ReflectedApparentTemperature']),
                                                                          ATemp=FlirImageExtractor.extract_float(
                                                                              meta['AtmosphericTemperature']),
                                                                          IRWTemp=FlirImageExtractor.extract_float(
                                                                              meta['IRWindowTemperature']),
                                                                          IRT=meta['IRWindowTransmission'],
                                                                          RH=FlirImageExtractor.extract_float(
                                                                              meta['RelativeHumidity']),
                                                                          PR1=meta['PlanckR1'], PB=meta['PlanckB'],
                                                                          PF=meta['PlanckF'],
                                                                          PO=meta['PlanckO'], PR2=meta['PlanckR2'])

        return thermal_np


    def raw2temp(self, raw, E=1, OD=1, RTemp=20, ATemp=20, IRWTemp=20, IRT=1, RH=50, PR1=21106.77, PB=1501, PF=1, PO=-7340,
                 PR2=0.012545258):
        """
        convert raw values from the flir sensor to temperatures in C
        # this calculation has been ported to python from
        # https://github.com/gtatters/Thermimage/blob/master/R/raw2temp.R
        # a detailed explanation of what is going on here can be found there
        """
        self.counter += 1

        # constants
        ATA1 = 0.006569
        ATA2 = 0.01262
        ATB1 = -0.002276
        ATB2 = -0.00667
        ATX = 1.9

        # transmission through window (calibrated)
        emiss_wind = 1 - IRT
        refl_wind = 0

        # transmission through the air
        h2o = (RH / 100) * exp(1.5587 + 0.06939 * (ATemp) - 0.00027816 * (ATemp) ** 2 + 0.00000068455 * (ATemp) ** 3)
        tau1 = ATX * exp(-sqrt(OD / 2) * (ATA1 + ATB1 * sqrt(h2o))) + (1 - ATX) * exp(
            -sqrt(OD / 2) * (ATA2 + ATB2 * sqrt(h2o)))
        tau2 = ATX * exp(-sqrt(OD / 2) * (ATA1 + ATB1 * sqrt(h2o))) + (1 - ATX) * exp(
            -sqrt(OD / 2) * (ATA2 + ATB2 * sqrt(h2o)))

        # radiance from the environment
        raw_refl1 = PR1 / (PR2 * (exp(PB / (RTemp + 273.15)) - PF)) - PO
        raw_refl1_attn = (1 - E) / E * raw_refl1
        raw_atm1 = PR1 / (PR2 * (exp(PB / (ATemp + 273.15)) - PF)) - PO
        raw_atm1_attn = (1 - tau1) / E / tau1 * raw_atm1
        raw_wind = PR1 / (PR2 * (exp(PB / (IRWTemp + 273.15)) - PF)) - PO
        raw_wind_attn = emiss_wind / E / tau1 / IRT * raw_wind
        raw_refl2 = PR1 / (PR2 * (exp(PB / (RTemp + 273.15)) - PF)) - PO
        raw_refl2_attn = refl_wind / E / tau1 / IRT * raw_refl2
        raw_atm2 = PR1 / (PR2 * (exp(PB / (ATemp + 273.15)) - PF)) - PO
        raw_atm2_attn = (1 - tau2) / E / tau1 / IRT / tau2 * raw_atm2

        raw_obj = (raw / E / tau1 / IRT / tau2 - raw_atm1_attn -
                   raw_atm2_attn - raw_wind_attn - raw_refl1_attn - raw_refl2_attn)

        # temperature from radiance
        temp_celcius = PB / np.log(PR1 / (PR2 * (raw_obj + PO)) + PF) - 273.15
        return temp_celcius

    @staticmethod
    def extract_float(dirtystr):
        """
        Extract the float value of a string, helpful for parsing the exiftool data
        :return:
        """
        digits = re.findall(r"[-+]?\d*\.\d+|\d+", dirtystr)
        return float(digits[0])

    def plot(self):
        """
        Plot the rgb + thermal image (easy to see the pixel values)
        :return:
        """
        rgb_np = self.get_rgb_np()
        thermal_np = self.get_thermal_np()

        plt.subplot(1, 2, 1)
        plt.imshow(thermal_np, cmap='hot')
        plt.subplot(1, 2, 2)
        plt.imshow(rgb_np)
        plt.show()

    def save_images(self, min=None, max=None, bytesIO=False):
        """
        Save the extracted images
        :return:
        """
        if (min is not None and max is None) or (max is not None and min is None):
            raise Exception('Specify both a maximum and minimum temperature value, or use the default by specifying neither')
        if max is not None and min is not None and max <= min:
            raise Exception("The max value must be greater than min")

        if self.thermal_image_np is None:
            self.thermal_image_np = self.extract_thermal_image()


        if min is not None and max is not None:
            thermal_normalized = ((self.thermal_image_np - min) / (max - min))
        else:
            thermal_normalized = ((self.thermal_image_np - np.amin(self.thermal_image_np)) / (np.amax(self.thermal_image_np) - np.amin(self.thermal_image_np)))

        if not bytesIO:
            thermal_output_filename_array = self.flir_img_filename.split(".")
            thermal_output_filename = thermal_output_filename_array[0] + "_thermal." + thermal_output_filename_array[1]

        return_array = []
        for palette in self.palettes:
            img_thermal = Image.fromarray(palette(thermal_normalized, bytes=True))
            # convert to jpeg and enhance
            img_thermal = img_thermal.convert("RGB")
            enhancer = ImageEnhance.Sharpness(img_thermal)
            img_thermal = enhancer.enhance(3)

            if bytesIO:
                bytes = io.BytesIO()
                img_thermal.save(bytes, "jpeg", quality=100)
                return_array.append(bytes)
            else:
                filename_array = thermal_output_filename.split(".")
                filename = filename_array[0] + "_" + str(palette.name) + "." + filename_array[1]
                if self.is_debug:
                    print("DEBUG Saving Thermal image to:{}".format(filename))

                img_thermal.save(filename, "jpeg", quality=100)
                return_array.append(filename)

        return return_array

    def export_thermal_to_csv(self, csv_filename):
        """
        Convert thermal data in numpy to csv
        :return:
        """
        with open(csv_filename, 'w') as fh:
            writer = csv.writer(fh, delimiter=',')

            pixel_values = [[]]
            current_x = 0
            for e in np.ndenumerate(self.thermal_image_np):
                x, y = e[0]

                if x > current_x:
                    pixel_values.append([])
                    current_x = x

                c = e[1].round(2)
                pixel_values[x].append(c)

            writer.writerows(pixel_values)

    @staticmethod
    def reject_outliers(data, m=2):
        return data[abs(data - np.mean(data)) < m * np.std(data)]
